<?php
/**
 * Theme setup class.
 *
 * Set up theme defaults and registers support for various WordPress features.
 *
 * Note that this function is hooked into the after_setup_theme hook, which
 * runs before the init hook. The init hook is too late for some features, such
 * as indicating support post thumbnails.
 *
 */

class The_Universe_Setup {

  // Theme version
  protected $version;
  
  // Theme unique id
  protected $theme_slug;
  
  public function __construct() {
    $this->version    = '1.2.1';
    $this->theme_slug = 'the-universe';
  }
  
  /**
   * Setup theme. Add actions, filters, features etc.
   *
   * @since The Universe 1.0
   */   
  public function setup() {
    global $content_width;

    /**
     * Set up the content width value based on the theme's design.
     */
    if ( ! isset( $content_width ) ) {
      $content_width = 1300;
    }
  
    /*
     * Make theme available for translation.
     *
     * Translations can be added to the /languages/ directory.
     */
    load_theme_textdomain( 'the-universe', get_template_directory() . '/languages' );
  
    // Add RSS feed links to <head> for posts and comments.
    add_theme_support( 'automatic-feed-links' );

    // Enable support for Post Thumbnails, and declare two sizes.
    add_theme_support( 'post-thumbnails' );
    set_post_thumbnail_size( 1300, 820, true );
    add_image_size( 'the-universe-full-width', 1300, 9999, false );
    add_image_size( 'the-universe-medium', 500, 330, true );
    add_image_size( 'the-universe-portfolio-grid', 750, 500, false );
    add_image_size( 'the-universe-header-image', 1920, 800, true );
    
    // Register navigation menus.
    register_nav_menus( array(
      'primary' => esc_html__( 'Primary Navigation', 'the-universe' )
    ) );
    
    /*
     * Switch default core markup for search form, comment form, and comments
     * to output valid HTML5.
     */
    add_theme_support( 'html5', array(
      'search-form', 'comment-form', 'comment-list', 'gallery', 'caption'
    ) );
    
    // Add document title tag to HTML <head>.
    add_theme_support( 'title-tag' );

    /*
     * Enable support for Post Formats.
     *
     * See: https://codex.wordpress.org/Post_Formats
     */
    add_theme_support( 'post-formats', array( 'video', 'audio', 'gallery' ) );
    add_post_type_support( 'page', 'post-formats' );
    
    // Setup the WordPress core custom header image.  
    add_theme_support( 'custom-header', array( 
      'width'         => 1920,
      'height'        => 800,
      'flex-height'   => true,
      'flex-width'    => true,
      'default-image' => get_template_directory_uri() . '/assets/images/header.jpg',
    ) );
    
    // Setup the WordPress core custom background feature.    
    add_theme_support( 'custom-background', array( 
      'default-color' => 'f9f9f9'
    ) );

    // Add theme support for Custom Logo.
    add_theme_support( 'custom-logo', array(
      'flex-width' => true,
    ) );
    
    // Add public scripts and styles
    add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
    add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ), 10 );
    add_action( 'wp_enqueue_scripts', array( $this, 'post_nav_background' ), 10 );
    
    // Widgets init
    add_action( 'widgets_init', array( $this, 'widgets_init' ) );
    
    // Modify Read More link
    add_filter( 'the_content_more_link', array( $this, 'modify_read_more_link' ) );
    
    // Modify search form
    add_filter( 'get_search_form', array( $this, 'search_form' ) );
    
    // Set new excerpt more text
    add_filter('excerpt_more', array( $this, 'custom_excerpt_more' ));
    
    // This theme uses its own gallery styles.
    add_filter( 'use_default_gallery_style', '__return_false' );
    
    // Add classes to body tag and posts.
    add_filter( 'body_class', array( $this, 'add_body_classes' ) );
    add_filter( 'post_class', array( $this, 'add_post_classes' ) );
    
    // Remove custom styles and texts transients when this updated
    add_action( 'customize_save_after', array( $this, 'reset_custom_styles_cache' ) );
    add_action( 'edited_category',      array( $this, 'reset_custom_styles_cache' ) );
    add_action( 'create_category',      array( $this, 'reset_custom_styles_cache' ) );
    add_action( 'customize_save_after', array( $this, 'reset_custom_texts_cache' ) );

    // Modification archives title
    add_filter( 'get_the_archive_title', array( $this, 'archive_title' ), 10, 1 );

    add_filter( 'wp_list_categories', array( $this, 'cat_count_span' ) );
    add_filter( 'get_archives_link',  array( $this, 'archive_count_span' ) );

    // Add support for Block Styles.
    add_theme_support( 'wp-block-styles' );

    add_theme_support( 'responsive-embeds' );
  }
  
  /**
   * Register and enqueue theme scripts.
   *
   * @since The Universe 1.0
   */  
  public function enqueue_scripts() {
    // Enqueue scripts
    if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
      wp_enqueue_script( 'comment-reply' );
    }

    // Owl Carousel
    wp_register_script( 'owl-carousel',      get_template_directory_uri() . '/assets/js/owl.carousel.min.js', array(), '2.2.1' );

    wp_enqueue_script( 'the-universe-functions', get_template_directory_uri() . '/assets/js/functions.dev.js', array( 'jquery' ), $this->version, true );     
  }
  
  /**
   * Register and enqueue theme styles.
   *
   * @since The Universe 1.0
   */    
  public function enqueue_styles() {
    // Add fonts, used in the main stylesheet.
    wp_enqueue_style( 'the-universe-fonts', $this->fonts_url(), array(), $this->version );
    
    // Font Awesome icons font style
    wp_register_style( 'font-awesome', get_template_directory_uri() . '/assets/css/font-awesome.min.css', array(), '4.7.0' );

    // Owl Carousel Style
    wp_register_style( 'owl-carousel', get_template_directory_uri() . '/assets/css/owl.carousel.min.css', array(), '2.2.1' );     
    // Load our main stylesheet.
    wp_enqueue_style( 'the-universe-style', get_stylesheet_uri(), array( 'font-awesome' ), $this->version );
    
    // Add customizer css.
    wp_add_inline_style( 'the-universe-style', $this->inline_style() );
  }
  
  /**
   * Register Google fonts for The Universe.
   *
   * @since The Universe 1.0
   *
   * @return string Google fonts URL for the theme.
   */  
  public function fonts_url() {
    $fonts_url = '';
    $fonts     = array();
    $subsets   = 'latin,latin-ext';

    /* translators: If there are characters in your language that are not supported by Nunito, translate this to 'off'. Do not translate into your own language. */
    if ( 'off' !== esc_attr_x( 'on', 'Nunito font: on or off', 'the-universe' ) ) {
      $fonts[] = 'Nunito:300,300italic,400,400italic,700,700italic';
    }

    /* translators: If there are characters in your language that are not supported by Source Serif Pro, translate this to 'off'. Do not translate into your own language. */
    if ( 'off' !== esc_attr_x( 'on', 'Source Serif Pro font: on or off', 'the-universe' ) ) {
      $fonts[] = 'Source Serif Pro:400,700';
    }

    /* translators: To add an additional character subset specific to your language, translate this to 'greek', 'cyrillic', 'devanagari' or 'vietnamese'. Do not translate into your own language. */
    $subset = esc_attr_x( 'no-subset', 'Add new subset (greek, cyrillic, devanagari, vietnamese)', 'the-universe' );

    if ( 'cyrillic' == $subset ) {
      $subsets .= ',cyrillic,cyrillic-ext';
    } elseif ( 'greek' == $subset ) {
      $subsets .= ',greek,greek-ext';
    } elseif ( 'devanagari' == $subset ) {
      $subsets .= ',devanagari';
    } elseif ( 'vietnamese' == $subset ) {
      $subsets .= ',vietnamese';
    }

    if ( $fonts ) {
      $fonts_url = esc_url( add_query_arg( array(
        'family' => urlencode( implode( '|', $fonts ) ),
        'subset' => urlencode( $subsets ),
      ), '//fonts.googleapis.com/css' ) );
    }

    return $fonts_url;
  }
  
  /**
   * Inline style. Used in wp_add_inline_style.
   *
   * @since The Universe 1.0
   */  
  public function inline_style() {
    if( is_customize_preview() )
      return $this->get_inline_style();
      
    $inline_style = get_transient( 'the_universe_inline_style' );
    
    if( $inline_style === false ) {
      $inline_style = $this->get_inline_style();
      set_transient( 'the_universe_inline_style', $inline_style );
    }

    if( $this->get_header_image_url() ) {
      $inline_style .= sprintf( '
        .site-header {
          background-image: url( %s )
        }',
        esc_url( $this->get_header_image_url() )
      );
    }
    
    return $inline_style;
  }
  
  /**
   * Get inline style CSS.
   *
   * @since The Universe 1.0
   *
   * @return string Css with custom styles.
   */  
  public function get_inline_style() {
    $css = '';

    $header_image_blackout = get_theme_mod( 'header_image_blackout' );
    
    if( is_numeric( $header_image_blackout ) && $header_image_blackout >=0 && $header_image_blackout <= 1 ){
      $css .= sprintf( '
        .has-header-image .site-header:before{
          opacity: %s;
        }',
        $header_image_blackout
      );
    }
    
    $logo_size = absint( get_theme_mod( 'logo_size' ) );
    
    if( $logo_size ){
      $css .= sprintf( '
        .custom-logo-link img{
          max-width: %dpx;
        }',
        $logo_size
      );
    }

    $link_color = $this->sanitize_hex_color( get_theme_mod( 'link_color' ) );
    
    if( $link_color ) {
      $css .= sprintf( '
        .page-content a,
        .taxonomy-description a,
        .entry-content a:not(.wp-block-button__link),
        .post-meta a,
        .author-bio a,
        .author-link,
        .logged-in-as a,
        .comment-content a,
        .comment-edit-link,
        .comment-navigation a {
          color: %s;
        }',
        $link_color
      );
    }
    
    $link_color_hover = $this->sanitize_hex_color( get_theme_mod( 'link_color_hover' ) );
    
    if( $link_color_hover ) {
      $css .= sprintf( '
        .page-content a:hover,
        .taxonomy-description a:hover,
        .entry-content a:not(.wp-block-button__link):hover,
        .post-meta a:hover,
        .author-bio a:hover,
        .author-link:hover,
        .logged-in-as a:hover,
        .comment-content a:hover,
        .comment-edit-link:hover,
        .comment-navigation a:hover {
          color: %s;
        }',
        $link_color_hover
      );
    }
    
    $category_label_color = $this->sanitize_hex_color( get_theme_mod( 'category_label_color' ) );
    
    if( $category_label_color ) {
      $css .= sprintf( '
        .cat-links a {
          background: %s;
        }',
        $category_label_color
      );
    }
    
    $categories_color = get_option( 'the_universe_categories_color' );
    
    if( is_array( $categories_color ) ) {
      foreach( $categories_color as $category_id => $category_color ) {
        if( $this->sanitize_hex_color( $category_color ) ) {
          $css .= sprintf( '
        .cat-links .category-%d { 
          background: %s 
        }
          ', 
          absint( $category_id ), 
          $category_color 
         );
        }
      }
    }
    
    $button_background_color = $this->sanitize_hex_color( get_theme_mod( 'button_background_color' ) );
    
    if( $button_background_color ) {
      $css .= sprintf( '
        button,
        input[type="button"],
        input[type="reset"],
        input[type="submit"] {
          background-color: %s;
        }',
        $button_background_color
      );
    }
    
    $button_background_color_hover = $this->sanitize_hex_color( get_theme_mod( 'button_background_color_hover' ) );
    
    if( $button_background_color_hover ) {
      $css .= sprintf( '
        button:hover,
        input[type="button"]:hover,
        input[type="reset"]:hover,
        input[type="submit"]:hover {
          background-color: %s;
        }',
        $button_background_color_hover
      );
    }
    
    $button_text_color = $this->sanitize_hex_color( get_theme_mod( 'button_text_color' ) );
    
    if( $button_text_color ) {
      $css .= sprintf( '
        button,
        input[type="button"],
        input[type="reset"],
        input[type="submit"] {
          color: %s;
        }',
        $button_text_color
      );
    }

    if( $this->get_header_image_url() && is_customize_preview() ) {
      $css .= sprintf( '
        .site-header {
          background-image: url( %s )
        }',
        esc_url( $this->get_header_image_url() )
      );
    }
    
    return $css;
  }
  
  /**
   * Sanitizes a hex color.
   *
   * @since The Universe 1.0
   */  
  public function sanitize_hex_color( $color ) {
    if ( '' === $color )
      return '';
      
    // 3 or 6 hex digits, or the empty string.
    if ( preg_match('|^#([A-Fa-f0-9]{3}){1,2}$|', $color ) )
      return $color; 
  }

  /**
   * Get the header image URL.
   *
   * @since The Universe 1.1
   */  
  public function get_header_image_url() {
    if( is_single() || is_page() ) {
      global $post;

      $header_image_id = get_post_meta( $post->ID, 'the_universe_header_image_id', true );
      $header_image_src = wp_get_attachment_image_src( $header_image_id, 'the-universe-header-image' );

      if( is_array( $header_image_src ) )
        return $header_image_src[0];
    }

    if( has_header_image() ) {
      $header_image_url = get_header_image();

      return $header_image_url;
    }
  }
  
  /**
   * Add featured image as background image to post navigation elements.
   *
   * @since The Universe 1.0
   *
   * @see wp_add_inline_style()
   */  
  function post_nav_background() {
    if ( ! is_single() ) {
      return;
    }

    $previous = ( is_attachment() ) ? get_post( get_post()->post_parent ) : get_adjacent_post( false, '', true );
    $next     = get_adjacent_post( false, '', false );
    $css      = '';

    if ( is_attachment() && 'attachment' == $previous->post_type ) {
      return;
    }

    if ( $previous &&  has_post_thumbnail( $previous->ID ) ) {
      $prevthumb = wp_get_attachment_image_src( get_post_thumbnail_id( $previous->ID ), 'post-thumbnail' );
      $css .= '
        .post-navigation .nav-previous { 
          background-image: url(' . esc_url( $prevthumb[0] ) . '); 
        }
        .post-navigation .nav-previous .post-title, 
        .post-navigation .nav-previous a:hover .post-title { 
          color: #fff; 
        }
        .post-navigation .nav-previous .nav-meta {
          color: #fff;
          border-color: rgba(255,255,255,0.2);
        }
        .post-navigation .nav-previous .nav-meta:hover {
          border-color: rgba(255,255,255,0.35);
        }
        .post-navigation .nav-previous:before { 
          background-color: rgba(0, 0, 0, 0.6); 
        }
        .post-navigation .nav-previous:hover:before { 
          background-color: rgba(0, 0, 0, 0.8); 
        }
      ';
    }

    if ( $next && has_post_thumbnail( $next->ID ) ) {
      $nextthumb = wp_get_attachment_image_src( get_post_thumbnail_id( $next->ID ), 'post-thumbnail' );
      $css .= '
        .post-navigation .nav-next { 
          background-image: url(' . esc_url( $nextthumb[0] ) . '); 
        }
        .post-navigation .nav-next .post-title, 
        .post-navigation .nav-next a:hover .post-title { 
          color: #fff; 
        }
        .post-navigation .nav-next .nav-meta {
          color: #fff;
          border-color: rgba(255,255,255,0.2);
        }
        .post-navigation .nav-next .nav-meta:hover {
          border-color: rgba(255,255,255,0.35);
        }
        .post-navigation .nav-next:before { 
          background-color: rgba(0, 0, 0, 0.6); 
        }
        .post-navigation .nav-next:hover:before { 
          background-color: rgba(0, 0, 0, 0.8); 
        }
      ';
    }

    wp_add_inline_style( 'the-universe-style', $css );
  }
  
  /**
   * Reset custom styles cache.
   *
   * @since The Universe 1.0
   */
  function reset_custom_styles_cache() {
    delete_transient( 'the_universe_inline_style' );
  }
  
  /**
   * Reset custom text cache.
   *
   * @since The Universe 1.0
   */
  function reset_custom_texts_cache() {
    delete_transient( 'the_universe_tagline' );
    delete_transient( 'the_universe_copyright' );
  }

  /**
   * Register widget area.
   *
   * @since The Universe 1.0
   *
   * @link https://codex.wordpress.org/Function_Reference/register_sidebar
   */
  public function widgets_init() {  
    // Register widgets
    register_sidebar( array(
      'name'          => esc_html__( 'Footer Widgets &mdash; Column 1', 'the-universe' ),
      'id'            => 'widget-area-1',
      'description'   => esc_html__( 'Add widgets here to appear in footer.', 'the-universe' ),
      'before_widget' => '<aside id="%1$s" class="widget %2$s">',
      'after_widget'  => '</aside>',
      'before_title'  => '<h3 class="widget-title">',
      'after_title'   => '</h3>',
    ) );

    register_sidebar( array(
      'name'          => esc_html__( 'Footer Widgets &mdash; Column 2', 'the-universe' ),
      'id'            => 'widget-area-2',
      'description'   => esc_html__( 'Add widgets here to appear in footer.', 'the-universe' ),
      'before_widget' => '<aside id="%1$s" class="widget %2$s">',
      'after_widget'  => '</aside>',
      'before_title'  => '<h3 class="widget-title">',
      'after_title'   => '</h3>',
    ) );

    register_sidebar( array(
      'name'          => esc_html__( 'Footer Widgets &mdash; Column 3', 'the-universe' ),
      'id'            => 'widget-area-3',
      'description'   => esc_html__( 'Add widgets here to appear in footer.', 'the-universe' ),
      'before_widget' => '<aside id="%1$s" class="widget %2$s">',
      'after_widget'  => '</aside>',
      'before_title'  => '<h3 class="widget-title">',
      'after_title'   => '</h3>',
    ) );
  }
  
  /**
   * More link modification.
   *
   * @since The Universe 1.0
   *
   * @return string Modified more link.
   */  
  public function modify_read_more_link() {
    $more_link_text = get_theme_mod( 'more_link_title' ) ? get_theme_mod( 'more_link_title' ) : esc_html__( 'Read More', 'the-universe' );
  
    return sprintf( '<p class="entry-more"><a class="more-link" href="%s">%s</a></p>', esc_url( get_permalink() ), esc_html( $more_link_text ) );
  }
  
  /**
   * Search form modification.
   *
   * @since The Universe 1.0
   *
   * @return string HTML with search form.
   */   
  public function search_form( $form ) {
  
    $form = '
      <form method="get" class="search-form" action="' . esc_url( home_url( '/' ) ) . '">
        <div class="search-wrap">
          <label>
            <input type="search" class="search-field" placeholder="' . esc_attr_x( 'Search &hellip;', 'placeholder', 'the-universe' ) . '" value="' . get_search_query() . '" name="s" title="' . esc_attr_x( 'Search for:', 'title tag', 'the-universe' ) . '" />
          </label>
          <button type="submit" class="search-submit"><i class="fa fa-search"></i></button>
        </div>
			</form>';
			
    return $form;
  }
  
  /**
   * Modification default excerpt more.
   *
   * @since The Universe 1.0
   *
   * @return string Excerpt more text.
   */   
  public function custom_excerpt_more( $more ) {
    return '...';
  }
  
   /**
   * Add classes to body tag.
   *
   * @since The Universe 1.0
   *
   * @return array Body classes array.
   */    
  public function add_body_classes( $classes ) {
    if( has_header_image() ) {
      $classes[] = 'has-header-image';
    } else if( is_single() || is_page() ) {
      
      global $post;

      $header_image_id = get_post_meta( $post->ID, 'the_universe_header_image_id', true );
      $header_image_src = wp_get_attachment_image_src( $header_image_id, 'the-universe-header-image' );

      if( is_array( $header_image_src ) )
        $classes[] = 'has-header-image';
    }

    if( get_theme_mod( 'display_sticky_menu', 1 ) ) {
      $classes[] = 'has-sticky-menu';
    }
          
    return $classes;
  }

  /**
   * Add classes to post.
   *
   * @since The Universe 1.1
   *
   * @return array Post classes array.
   */    
  public function add_post_classes( $classes ) {
    $classes[] = 'post-wrap';
          
    return $classes;
  }
   
  /**
   * Remove () and add <span> tag to posts count in categories list.
   *
   * @since The Universe 1.0
   *
   * @return string Html with links
   */      
  function cat_count_span( $links ) {
    $links = str_replace( '</a> (', '</a> <span>', $links );
    $links = str_replace( ')', '</span>', $links );
    return $links;
  }
  
  /**
   * Remove () and add <span> tag to posts count in archives list.
   *
   * @since The Universe 1.0
   *
   * @return string Html with links
   */   
  function archive_count_span( $link ) {
    $link = str_replace( '</a>&nbsp;(', '</a> <span>', $link );
    $link = str_replace( ')</li>', '</span></li>', $link );
    return $link;
  }

  /**
   * Changing default archive title for portfolio post type.
   *
   * @since The Universe 1.0
   */    
  public function archive_title( $title ) {
    if ( is_category() ) {
      return single_cat_title( '', false );
    }

    if ( is_author() ) {
      return get_the_author();
    }

    return $title;
  }
}

$the_universe_setup = new The_Universe_Setup();

add_action( 'after_setup_theme', array( $the_universe_setup, 'setup' ), 10 );

/**
 * Customizer additions.
 *
 * @since The Universe 1.0
 */
require get_template_directory() . '/inc/customizer.php';

/**
 * Custom template tags for this theme.
 *
 * @since The Universe 1.0
 */
require get_template_directory() . '/inc/template-tags.php';

/**
 * Add fields to category form.
 *
 * @since The Universe 1.0
 */
require get_template_directory() . '/inc/class-the-universe-category-form-fields-setup.php';

/**
 * Post Formats.
 *
 * @since The Universe 1.1
 */
require get_template_directory() . '/inc/class-the-universe-post-formats.php';

/**
 * Theme Metaboxes
 *
 * @since The Universe 1.1
 */
require get_template_directory() . '/inc/class-the-universe-metaboxes-setup.php';

/**
 * TGM Plugin Activation
 *
 * @since The Universe 1.2
 */
require get_template_directory() . '/inc/tgmpa/tgmpa.php';