<?php
/**
 * Custom template tags for The Universe
 *
 * Eventually, some of the functionality here could be replaced by core features.
 *
 * @since The Universe 1.0
 */

if ( ! function_exists( 'the_universe_site_identity' ) ) :
  /**
   * Prints HTML with blog title and description.
   *
   * @since The Universe 1.0
   */
  function the_universe_site_identity() {
    if( get_theme_mod( 'custom_logo' ) ) {
      the_universe_custom_logo();
    }

    if( get_theme_mod( 'display_site_title', 1 ) ) {
      printf( '<%1$s class="site-title"><a href="%2$s" rel="home">%3$s</a></%1$s>', 
        is_front_page() && is_home() ? 'h1' : 'p', 
        esc_url( home_url( '/' ) ), 
        esc_html( get_bloginfo( 'name' ) ) 
      );
    }

    the_universe_tagline( '<div class="tagline">', '</div>' );
  }
endif;

if ( ! function_exists( 'the_universe_custom_logo' ) ) :
  /**
   * Displays the optional custom logo.
   *
   * Does nothing if the custom logo is not available.
   *
   * @since The Universe 1.0
   */
  function the_universe_custom_logo() {
    if ( function_exists( 'the_custom_logo' ) ) {
      the_custom_logo();
    }
  }
endif;

if ( ! function_exists( 'the_universe_tagline' ) ) :
  /**
   * Prints HTML with custom header text.
   *
   * @since The Universe 1.0
   */
  function the_universe_tagline( $before = '', $after = '' ) {
    $header_text = get_transient( 'the_universe_tagline' );
    
    if( $header_text === false || is_customize_preview() ) {
      $header_text = get_theme_mod( 'tagline' );
      
      if( $header_text ) {
        $header_text = wp_kses( $header_text, array(
            'a' => array(
              'href' => array(),
              'title' => array()
            ),
            'p'      => array(),
            'b'      => array(),
            'strong' => array(),
            'em'     => array(),
            'i'      => array(),
            'br'     => array(),
            'span'   => array(),
            'img' => array(
              'src' => array(),
              'alt' => array(),
              'title' => array()
            )
          )
        );
      }
      set_transient( 'the_universe_tagline', $header_text );
    }
    
    printf( '%s%s%s', $before, $header_text, $after );
  }
endif;

if ( ! function_exists( 'the_universe_site_copyright' ) ) :
  /**
   * Prints HTML with custom or default copyright text.
   *
   * @since The Universe 1.0
   */
  function the_universe_site_copyright( $before = '', $after = '' ) {
    $default_copyright = sprintf( '&copy; %1$d <a href="%2$s">%3$s</a>', date('Y'), esc_url( home_url( '/' ) ), esc_html( get_bloginfo( 'name' ) ) );
    
    $copyright = get_transient( 'the_universe_copyright' );
    
    if( $copyright === false || is_customize_preview() ) {
      $copyright = get_theme_mod( 'copyright' );
      
      if( $copyright ) {
        $copyright = wp_kses( $copyright, array(
            'a' => array(
              'href' => array(),
              'title' => array()
            ),
            'p'      => array(),
            'b'      => array(),
            'strong' => array(),
            'em'     => array(),
            'i'      => array(),
            'br'     => array(),
            'span'   => array(),
            'img' => array(
              'src' => array(),
              'alt' => array(),
              'title' => array()
            )
          )
        );
      }
      set_transient( 'the_universe_copyright', $copyright );
    }
    
    printf( '%s%s%s', $before, $copyright ? $copyright : $default_copyright, $after );
  }
endif;

if ( ! function_exists( 'the_universe_social_profiles' ) ) :
  /**
   * Prints HTML with social profiles buttons.
   *
   * @since The Universe 1.0
   */
  function the_universe_social_profiles( $before = '', $after = '' ) {
    $social_profiles = get_theme_mod( 'social' );
    
    if( empty( $social_profiles ) )
      return;
    
    $output = '';
    
    if( is_array( $social_profiles ) ) {
      foreach( $social_profiles as $network_id => $url ) {
        if( $url )
          $output .= sprintf( 
            '<li><a href="%1$s" class="%2$s"><i class="fa fa-%2$s"></i></a></li>', 
            ( $network_id == 'email' ) ? 'mailto:' . esc_attr( $url ) : esc_url( $url ), 
            esc_attr( $network_id ) 
         );
      }
    }
    
    if( $output )
      printf( '%s<ul class="social-profiles">%s</ul>%s', $before, $output, $after );
  }
endif;

if ( ! function_exists( 'the_universe_post_thumbnail' ) ) :
  /**
   * Display an optional post thumbnail.
   *
   * Wraps the post thumbnail in an anchor element on index views, or a div
   * element when on single views.
   *
   * @since The Universe 1.0
   */
  function the_universe_post_thumbnail() {
    if ( post_password_required() || is_attachment() || ! has_post_thumbnail()  ) {
      return;
    }

    if ( is_singular() ) :
    ?>

    <div class="post-thumbnail">
      <?php the_post_thumbnail(); ?>
    </div><!-- .post-thumbnail -->

    <?php else : ?>

    <a class="post-thumbnail" href="<?php the_permalink(); ?>" aria-hidden="true">
      <?php the_post_thumbnail( 'post-thumbnail', array( 'alt' => get_the_title() ) ); ?>
    </a>

    <?php endif; // End is_singular()
  }
endif;

if ( ! function_exists( 'the_universe_entry_meta' ) ) :
  /**
   * Prints HTML with meta information for the categories, post date.
   *
   * @since The Universe 1.0
   */
  function the_universe_entry_meta( $before = '', $after = '' ) {
    $output = '';
    $post_type = get_post_type();

    if ( in_array( $post_type, array( 'post', 'attachment' ) ) && get_theme_mod( 'display_date', 1 ) ) {
      $time_string = sprintf( '<time class="entry-date published" datetime="%1$s">%2$s</time>',
        esc_attr( get_the_date( 'c' ) ),
        get_the_date()
      );

      $output .= sprintf( '<span class="posted-on post-meta">%s</span>', sprintf( esc_html__( 'Posted on %s', 'the-universe' ), $time_string ) );
    }

    if ( 'post' == $post_type && get_theme_mod( 'display_author', 1 ) ) {
      $author_string = sprintf( '<a class="url fn" href="%s">%s</a>', esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ), get_the_author() );
      $author_string = sprintf( esc_html_x( 'by %s', 'Post author', 'the-universe' ), $author_string );
    
      $output .= sprintf( '<span class="byline author vcard post-meta">%s</span>', $author_string );
    }

    if( $output )
      printf( '%s%s%s', $before, $output, $after );
  }
endif;

if ( ! function_exists( 'the_universe_category_list' ) ) :
  /**
   * Prints HTML with post categories list.
   *
   * @since The Universe 1.0
   */
  function the_universe_category_list() {
    $post_type = get_post_type();
    
    if ( 'post' == $post_type && get_theme_mod( 'display_categories', 1 ) ) {
      $categories = get_the_category();
      $output = '';
    
      if( $categories ) {
        foreach( $categories as $category ) {
          $output .= sprintf( '<a href="%s" class="category-%d">%s</a>', esc_url( get_category_link( $category->term_id ) ), $category->term_id, esc_attr( $category->cat_name ) );
        }
      }
    
      if( $output ) {
        printf( '<div class="cat-links">%s</div>', $output );
      }  
    }
  }
endif;

if ( ! function_exists( 'the_universe_entry_footer' ) ) :
  /**
   * Prints HTML with post tags and share buttons in post footer
   *
   * @since The Universe 1.0
   */
  function the_universe_entry_footer( $display_tags = true, $display_share_buttons = true, $before = '', $after = '' ) {
    $output = '';
    $post = get_post();
    
    if ( is_singular( 'post' ) && $display_tags ) {
      $tags_list = get_the_tag_list( '<div class="tags-list">',' ','</div>' );
    
      if ( $tags_list  ) 
        $output .= $tags_list;
    }

    if( is_singular() && $display_share_buttons && function_exists( 'usmb_get_share_buttons' ) ) {
      $output .= sprintf( '<div class="entry-share">%s</div>', usmb_get_share_buttons() );
    }
    
    if( $output )
      printf( '%s%s%s', $before, $output, $after );
  }
endif;

if ( ! function_exists( 'the_universe_related_posts' ) ) :
  /**
   * Related posts for current post.
   *
   * @since The Universe 1.0
   */
  function the_universe_related_posts( $before = '', $after = '' ) {
    global $post;
    
    if( !$post )
      return;

    if( $post->post_type !== 'post' )
      return;
      
    $query_args = array(
      'post__not_in'        => array( $post->ID ),
      'posts_per_page'      => 3,
      'orderby'             => 'rand',
      'ignore_sticky_posts' => true
    );
    
    $related_by = get_theme_mod( 'related_by', 'none' );
    
    switch ( $related_by ) {
      case 'category' :
        $categories = get_the_category( $post->ID );
        if ( $categories ) {
          $category_ids = array();
          foreach( $categories as $category ) 
            $category_ids[] = $category->term_id;
            
          $query_args['category__in'] = $category_ids;
        }
        break;
      
      case 'tags' :
        $tags = wp_get_post_tags( $post->ID );
        if ( $tags ) {
          $tag_ids = array();
          foreach( $tags as $tag ) 
            $tag_ids[] = $tag->term_id;
            
          $query_args['tag__in'] = $tag_ids;
        }
        break;
      case 'none' :
        $query_args['orderby'] = 'rand';
        break;
    }
    
    $related_posts = new WP_Query( $query_args );
    $output = '';
    
    if( $related_posts->have_posts() ) {
      printf( '%s<div class="related-posts">', $before );
      
      while( $related_posts->have_posts() ) : $related_posts->the_post(); ?>
      <div class="related-post">
        <?php if( has_post_thumbnail() ) : ?>
        <a href="<?php the_permalink()?>" class="post-thumbnail" rel="bookmark" title="<?php the_title(); ?>">
          <?php the_post_thumbnail( 'the-universe-medium' ); ?>
        </a>
        <?php endif; ?>
        
        <div class="related-content">
          <a href="<?php the_permalink(); ?>" class="related-post-title" rel="bookmark" title="<?php the_title(); ?>"><?php the_title(); ?></a>
        </div>
      </div>
    <?php 
      endwhile;
      
      printf( '</div>%s', $after );
    }

    wp_reset_postdata(); 
  }
endif;
?>